require 'spec_helper'

describe ModFactor::Owner do

	with_model :financial_factor do
		table do |t|
			t.string :key
			t.decimal :amount
			t.string :owner_type
			t.integer :owner_id
		end
		model do
			belongs_to :owner, :polymorphic => true
		end
	end

	with_model :preference do 
		table do |t|
			t.string :key
			t.string :owner_type
			t.string :owner_id
			t.text :value
			t.timestamps
		end
		model do 
			belongs_to :owner, :polymorphic => true
		end
	end

	with_model :family do
		table do |t|
			t.boolean "visible"
		end
		model do
			include ModFactor::Owner

			has_many :family_members
			has_many :members, :through=>:family_members

			factors :financial_factors do
				factor :prop_one
				factor :prop_store
				factor :prop_formula, :generate => true, 
					:formula => proc {|extra| 1 + 1}
				factor :prop_one_times_ten, :generate=>true, 
					:formula => proc {|extra| (prop_one||0) * 10}
				factor :prop_child, :generate => true,
					:formula => proc {|extra| members.first.try :prop_member}
				factor :prop_extra, :generate => true,
					:formula => proc {|extra| extra[:extra_value]}
				factor :prop_condition, :generate => true, :condition => proc {|extra| prop_one == 10},
					:formula => proc {|extra| prop_one || raise("prop one is nil")}
				factor :prop_default, :default=>10
			end

			properties :preferences do
				property :pref_boolean, :boolean, :default=>false
				property :pref_name, :string, :default=>'name'
			end

			validates :prop_one, :presence=>true
		end
	end

	with_model :family_member do
		table do |t|
			t.references :family
			t.references :member
		end
		model do
			belongs_to :family
			belongs_to :member
		end
	end

	with_model :member do
		table do |t|
			t.string :name
		end
		model do
			has_many :family_members
			has_many :family, :through=>:family_member
			include ModFactor::Owner

			factors :financial_factors do
				factor :prop_member
			end
		end
	end

	subject { Family.create! :prop_one => 5 }

	it "should has define factors" do
		subject.respond_to? :financial_factors
	end

	it "should access factor" do
		subject.prop_one = 10
		subject.prop_one.should == 10 
	end

	it "should store factor in factors table" do
		expect {
			subject.prop_store = 10
			subject.save!
		}.to change {subject.financial_factors.size}
	end

	it "should generate factor after calculation" do
		expect {
			subject.calculate
		}.to change {subject.prop_formula}
	end

	it "should ten time prop_one to prop_one_time_ten" do
		subject.prop_one = 10
		subject.calculate
		subject.prop_one_times_ten.should == 100
	end

	it "should proper store generate factor" do
		subject.prop_one = 10
		subject.calculate
		subject.save!
		Family.find(subject.id).prop_formula.should == 2
	end

	it "should load child factor" do
		member = subject.members.create!
		member.prop_member = 10
		member.save!
		subject.calculate
		subject.members.first.prop_member.should == 10
		subject.prop_child.should == 10
	end

	it "should work with calculate options" do
		subject.calculate :extra_value => 10
		subject.prop_extra.should == 10
	end

	it "should meet condition before generate" do
		subject.calculate
		subject.prop_condition.should == nil
		subject.prop_one = 10
		subject.calculate
		subject.prop_condition.should == 10
	end

	it "should validate false when require factor blank" do
		subject.prop_one = nil
		subject.should_not be_valid
	end

	it "should have default value" do
		subject.prop_default.should == 10
	end

	it "should convert attribute" do
		subject.prop_one = "10"
		subject.save!
		subject.prop_one.should == 10
	end

	it "should changed for autosave" do
		member = subject.members.create!
		member.prop_member = 10
		subject.should be_changed_for_autosave
		member.should be_changed_for_autosave
	end

	it "should write and read property" do
		subject.pref_boolean = true
		subject.pref_boolean.should be_true
	end

	it "should store property to database" do
		subject.pref_boolean = true
		expect {
			subject.save!
		}.to change {subject.preferences.size}
	end

	it "should have default property" do
		subject.pref_name.should == 'name'
	end

end
